﻿using System;
using System.Drawing;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win.UltraWinChart;
using Infragistics.UltraChart;
using Infragistics.UltraChart.Shared;
using Infragistics.UltraChart.Shared.Styles;
using System.IO;
using System.Reflection;
using System.Drawing.Imaging;

namespace WinChartinWinGrid
{
    public class ChartInsideGridCellDrawFilter : IUIElementDrawFilter
    {
        #region Members
        //Reference to the Chart
        UltraChart _theChart;
        //Reference to the Grid
        private UltraGrid _theGrid;

        public delegate void ChartInfoRequestedDelegate(object Sender, ChartInfoEventArgs e);

        public event ChartInfoRequestedDelegate ChartInfoRequested;
        #endregion

        #region Constructor
        public ChartInsideGridCellDrawFilter(UltraGrid theGrid)
        {
            _theGrid = theGrid;
            _theChart = new UltraChart();

            theGrid.FindForm().Controls.Add(_theChart);

            _theGrid.MouseEnterElement += new UIElementEventHandler(_theGrid_MouseEnterElement);

            _theChart.MouseLeave += new EventHandler(_theChart_MouseLeave);
        }

        #endregion
        #region Props

        //Reference to the Current WinGrid Cell:
        private UltraGridCell _currentCell = null;
        public UltraGridCell CurrentCell
        {
            get { return _currentCell; }
            set { _currentCell = value; }
        }

        //Reference to the Chart
        public UltraChart Chart
        {
            get { return _theChart; }
        }

        #endregion

        #region Grid Events

        private void _theGrid_MouseEnterElement(object sender,
            Infragistics.Win.UIElementEventArgs e)
        {
            this.ShowChartInCell(e.Element, _theGrid);
        }

        #endregion
        #region Chart Events
        private void _theChart_MouseLeave(object sender, EventArgs e)
        {
            this.CurrentCell.Tag = HelperFunctions.ChartToLayout(this.Chart);

            this.Chart.Visible = false;
        }
        #endregion

        #region Methods

        private void ShowChartInCell(
            UIElement theUIElement, UltraGrid theGrid)
        {
            //Only target the following element type:
            if (theUIElement is EditorWithTextDisplayTextUIElement)
            {
                //Get the WinGrid cell associated with this UIElement
                UltraGridCell theCell = theUIElement.GetContext(typeof(UltraGridCell)) as UltraGridCell;

                //Make sure that we are on the "CHART" cell
                //that is supposed to receive the Chart component:
                if (theCell != null &&
                    theCell.Column.Key == "CHART" &&
                    theCell.Tag != null)
                {
                    this.CurrentCell = theCell; //Reference the Current Cell

                    Rectangle r = theUIElement.Rect;

                    //Determines if the clipped rectangle should be used
                    if (theUIElement.ClipRect.Width < theUIElement.Rect.Width)
                    {
                        r = theUIElement.ClipRect;
                    }

                    //We use the SetBounds method of the Chart to
                    //position and size it directly over the
                    //UIElement that represents the Current Cell
                    this.Chart.SetBounds(
                        r.Location.X + theGrid.Location.X,
                        r.Location.Y + theGrid.Location.Y,
                        r.Width,
                        r.Height);

                    //Get the ChartLayout object that is stored
                    //inside the CurrentCell's Tag and apply
                    //its property values to the Chart
                    HelperFunctions.LayoutToChart(
                        this.CurrentCell.Tag as ChartLayout,
                        this.Chart);

                    //Show the Chart, Bring it to the Front:
                    this.Chart.Visible = true;
                    this.Chart.BringToFront();
                    this.Chart.Focus();
                    this.Chart.Refresh();
                }
            }
        }
        protected void onChartInfoRequested(ChartInfoEventArgs e)
        {
            if (ChartInfoRequested != null)
            {
                ChartInfoRequested(this, e);
            }
            else
            {
                throw new ApplicationException(
                    "The ChartInfoRequested event must be handled otherwise the Chart will not render");
            }
        }

        #endregion
        #region IUIElementDrawFilter Members

        public Infragistics.Win.DrawPhase GetPhasesToFilter(
            ref UIElementDrawParams drawParams)
        {
            return DrawPhase.BeforeDrawBackColor; //we will want to take over the phase
            //that happens when it is time to draw an element's backColor.
        }
        public bool DrawElement(Infragistics.Win.DrawPhase drawPhase,
        ref UIElementDrawParams drawParams)
        {
            //TRUE: I will handle the drawing myself; 
            //FALSE: let the control draw the element
            bool retVal = false;

            //Only target the following element type:
            if (drawParams.Element is EditorWithTextDisplayTextUIElement)
            {
                Graphics g = drawParams.Graphics;

                //Get the WinGrid cell associated with this UIElement
                UltraGridCell theGridCell =
                    drawParams.Element.GetContext(typeof(UltraGridCell)) as UltraGridCell;

                //Make sure that we are on the CHART cell / column
                if (theGridCell != null && theGridCell.Column.Key == "CHART")
                {
                    ChartLayout theChartLayout = theGridCell.Tag as ChartLayout;

                    Rectangle r = drawParams.Element.Rect;

                    //Look at the Cell's Tag for a chart image if it exists
                    if (theChartLayout == null)
                    {
                        //Create a new EventArg 
                        ChartInfoEventArgs e =
                            new ChartInfoEventArgs(_theChart, theGridCell.Row);

                        //Fire the Chart Info Requested event, exposing
                        //the Chart and current Grid Row to the
                        //developer
                        this.onChartInfoRequested(e);

                        //Determines if the clipped rectangle should be used
                        if (drawParams.Element.ClipRect.Width < drawParams.Element.Rect.Width)
                        {
                            r = drawParams.Element.ClipRect;
                        }
                        else
                        {
                            r = drawParams.Element.Rect;
                        }

                        //Position and set where the Chart will show
                        //which should be in the same place as the
                        //current Cell's UIElement Rectangle
                        _theChart.SetBounds(r.X,
                            r.Y,
                            r.Width,
                            r.Height);

                        _theChart.Data.DataBind();

                        //Copy the Chart's properties into the Layout object
                        //so that we can restore the Chart to these settings
                        //later on
                        theChartLayout = HelperFunctions.ChartToLayout(_theChart);

                        //Store the Layout in the Cell's Tag
                        theGridCell.Tag = theChartLayout;
                    }

                    //System.Diagnostics.Debug.WriteLine("DrawFilter: " + theGridCell.Row.Cells["CustomerID"].Value.ToString() );

                    //either way, we get our hands on a chart Image
                    //and then use the current UIElement's rectangle
                    //and we then draw the image into that rectangle

                    // g.DrawImage(theChartLayout.Image, r);\\Kmod

                    retVal = true;
                    // added to export the data to the Excel, since UIElement is not inside a cell of the grid


                    theGridCell.Appearance.Image = theChartLayout.Image;
                    theGridCell.Appearance.ImageHAlign = HAlign.Center;
                    theGridCell.Appearance.ImageVAlign = VAlign.Top;
                    //theGridCell.Appearance.ImageBackgroundStyle = ImageBackgroundStyle.Centered;
                    //theGridCell.Appearance.ImageBackground.
                }

            }

            return retVal;
        }

        #endregion


    }
    public class ChartInfoEventArgs : EventArgs
    {
        private UltraChart _chart = null;
        private UltraGridRow _row = null;

        public ChartInfoEventArgs(UltraChart Chart, UltraGridRow Row)
        {
            _chart = Chart;
            _row = Row;
        }

        public UltraChart WinChart
        {
            get { return _chart; }
        }

        public UltraGridRow Row
        {
            get { return _row; }
        }
    }
    public class ChartLayout
    {
        public float X;
        public float Y;
        public float Z;
        public float Scale;

        ChartType _ChartType;
        public ChartType ChartType
        {
            get { return _ChartType; }
            set { _ChartType = value; }
        }

        object _DataSource;
        public object DataSource
        {
            get { return _DataSource; }
            set { _DataSource = value; }
        }

        Image _Image;
        public Image Image
        {
            get { return _Image; }
            set { _Image = value; }
        }

        Infragistics.UltraChart.Resources.Appearance.ColorAppearance _ColorAppearance;
        public Infragistics.UltraChart.Resources.Appearance.ColorAppearance ColorModel
        {
            get { return _ColorAppearance; }
            set { _ColorAppearance = value; }
        }

        string _TitleTop;
        public string TitleTop
        {
            get { return _TitleTop; }
            set { _TitleTop = value; }
        }
    }

    public class HelperFunctions
    {
        public static ChartLayout ChartToLayout(UltraChart theChart)
        {
            ChartLayout theLayout = new ChartLayout();

            theChart.Data.DataBind();

            theLayout.ChartType = theChart.ChartType;
            theLayout.DataSource = theChart.Data.DataSource;

            theLayout.X = theChart.Transform3D.XRotation;
            theLayout.Y = theChart.Transform3D.YRotation;
            theLayout.Z = theChart.Transform3D.ZRotation;
            theLayout.Scale = theChart.Transform3D.Scale;

            theLayout.TitleTop = theChart.TitleTop.Text;

            MemoryStream m = new MemoryStream();

            theChart.SaveTo(m, ImageFormat.Bmp);

            theLayout.Image = Image.FromStream(m);

            return theLayout;
        }
        public static void LayoutToChart(ChartLayout theLayout, UltraChart theChart)
        {
            theChart.TitleTop.Text = theLayout.TitleTop;

            theChart.Transform3D.XRotation = theLayout.X;
            theChart.Transform3D.YRotation = theLayout.Y;
            theChart.Transform3D.ZRotation = theLayout.Z;
            theChart.Transform3D.Scale = theLayout.Scale;

            theChart.Data.DataSource = theLayout.DataSource;
            theChart.Data.DataBind();
        }

    }

}
